lexer grammar Silk;
options {
  language=Java;

}

@members {

private static Logger _logger = Logger.getLogger(SilkLexer.class);

private SilkLexerState lexerContext = new SilkLexerState();

private void transit(Symbol token) { lexerContext.transit(token); } 
private boolean isKey() { return lexerContext.isInKey(); }
private boolean isValue() { return lexerContext.isValue(); }
private boolean isInValue() { return lexerContext.isInValue(); }
private boolean isOutValue() { return lexerContext.isOutValue(); }
private boolean isHead() { return getCharPositionInLine() == 0; }

  public void resetContext() { lexerContext.reset(); }

  @Override
  public void reportError(RecognitionException e) {
        String message = "line=" + getLine() + "(" + getCharPositionInLine() + "): " + e.toString();;
        throw new XerialError(XerialErrorCode.INVALID_TOKEN, message);
  } 
  
  public static String sanitizeDataLine(String line)
  {
      line = line.trim();
      if (line.startsWith("\\"))
          return line.substring(1);
      else
          return line;
  }
  
  
}
@header {
/*--------------------------------------------------------------------------
 *  Copyright 2009 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// SilkLexer.java
// Since: Jan 28, 2009 1:00:02 PM
//
//--------------------------------------

package org.xerial.silk.impl;
import org.xerial.core.XerialError;
import org.xerial.core.XerialErrorCode;
import org.xerial.util.log.Logger;
import org.xerial.silk.impl.SilkLexerState;
import org.xerial.silk.impl.SilkLexerState.State;
import org.xerial.silk.impl.SilkLexerState.Symbol;
}

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 153
fragment
WhiteSpace
	:	(' ' | '\t') 
	;		

// comment 
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 159
LineComment: '#' ~(LineBreakChar)* ;
// Preamble: { isHead() }? => '%' ~(LineBreakChar)* ; 

// r: <CR> n : <LF>
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 163
fragment LineBreakChar: '\n' | '\r';

/*
LineBreak
	: ('\r' '\n' | '\r' | '\n' ) 
	{ $channel=HIDDEN; resetContext(); }
	;
	*/

// MultiLineSeparator: { isHead() }? => '--' WhiteSpace*; 
// MultiLineEntrySeparator: { isHead() }? => '>>' WhiteSpace*;
 	
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 175
NodeIndent: { isHead() }? => (' ')* '-' { transit(Symbol.NodeStart); } ;
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 176
FunctionIndent: { isHead() }? => (' ')* '@' { transit(Symbol.NodeStart); } ;
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 177
BlankLine: { isHead() }? => WhiteSpace*;

// fragment DataLineBody: ~('-' | '%' | '#' | '@' | WhiteSpace) ~('#' | LineBreakChar)*;
// DataLine: { isHead() }? 
//	=> WhiteSpace* DataLineBody LineComment? { setText(sanitizeDataLine($DataLineBody.text)); };

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 183
LParen: '(' { transit(Symbol.EnterParen); };  
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 184
RParen:	')' { transit(Symbol.LeaveParen); };
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 185
Comma: 	','; 
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 186
Colon:	':' { transit(Symbol.Colon); } ;


// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 189
Seq: 	'>';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 190
TabSeq:	'|';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 191
Star: 	'*';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 192
At:		'@' { transit(Symbol.At); };
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 193
Plus:	'+';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 194
LBracket:	'[' { transit(Symbol.EnterParen); };
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 195
RBracket:	']';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 196
Question:	'?';

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 198
fragment Digit: '0' .. '9';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 199
fragment Letter: 'A' .. 'F' | 'a' .. 'f';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 200
fragment HexDigit: Digit | Letter;
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 201
fragment UnicodeChar: ~('"'| '\\');
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 202
fragment EscapeSequence
	: '\\' ('\"' | '\\' | '/' | 'b' | 'f' | 'n' | 'r' | 't' | 'u' HexDigit HexDigit HexDigit HexDigit)
	; 
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 205
fragment StringChar :  UnicodeChar | EscapeSequence;
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 206
fragment StringChar_s: StringChar*;
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 207
String: '"' s=StringChar_s '"' { setText($s.text);  transit(Symbol.LeaveValue); };
 

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 210
fragment ScopeIndicator: '(' | ')';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 211
fragment FlowIndicator:  '[' | ']' | '{' | '}';
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 212
fragment Indicator:  FlowIndicator | ScopeIndicator | ',' | ':' | '#' | '>' | '|' | '*' | '\'' | '"' | '@' | '%' | '\\';	


// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 215
fragment PlainUnsafeChar: '"'| '\\' | '#' ;

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 217
fragment PlainSafeKey: ~(PlainUnsafeChar | ScopeIndicator | FlowIndicator | ',' | ':' | '>' | '*' | '|'); 
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 218
fragment PlainSafeIn: ~(PlainUnsafeChar | ScopeIndicator | ',');
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 219
fragment PlainSafeOut: ~(PlainUnsafeChar);

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 221
fragment NonSpaceChar: ~('"'| '\\' | WhiteSpace );
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 222
fragment PlainFirst
	:  ~('-' | '+' | '?' | PlainUnsafeChar | WhiteSpace | Indicator ) 
	| { isValue() }? => ('-' | '+' | (':' | '?') NonSpaceChar)
	;

// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 227
PlainOneLine
	: { isKey() }? =>  PlainFirst PlainSafeKey* { transit(Symbol.LeaveValue); }  
  | { isInValue() }? => PlainFirst  PlainSafeIn* { transit(Symbol.LeaveValue); }  
  | { isOutValue() }? => PlainFirst  PlainSafeOut* { transit(Symbol.LeaveValue); }  
	;


// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 234
JSON
	: { isValue() }? => '{'
	{
		//_logger.info("enter JSON object");
		InLineJSONLexer l = new InLineJSONLexer(input);
		CommonTokenStream tokens = new CommonTokenStream(l);
		InLineJSONParser p = new InLineJSONParser(tokens);
		p.jsonObjectFragment();
		
		//$channel = JSON_CHANNEL;
		emit(new CommonToken(JSON, getText())); 
	}
	| { isValue() }? => '['
	{
		//_logger.info("enter JSON array");
		InLineJSONLexer l = new InLineJSONLexer(input);
		CommonTokenStream tokens = new CommonTokenStream(l);
		InLineJSONParser p = new InLineJSONParser(tokens);
		p.jsonArrayFragment();
		
		//$channel = JSON_CHANNEL;
		emit(new CommonToken(JSON, getText())); 
	}  
	;
	 
// $ANTLR src "c:\Users\leo\work\eclipse\workspace\xerial\xerial-core\src\main\java\org\xerial\silk\impl\Silk.g" 259
Separation: { !isHead() }? => WhiteSpace+ { $channel=HIDDEN; };



// parser rules	


